package be.iminds.ilabt.jfed.restapi.debugdata;

import be.iminds.ilabt.jfed.log.ApiCallDetails;
import ch.qos.logback.classic.pattern.ThrowableProxyConverter;
import ch.qos.logback.classic.spi.ILoggingEvent;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nullable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class LogLineDDBuilder {
    private static final Logger LOG = LoggerFactory.getLogger(LogLineDDBuilder.class);

    private String level;
    private Timestamp date;
    private String thread;
    private String loggerName;
    private String message;
    private String stacktrace;

    public LogLineDDBuilder() {
    }

    public LogLineDDBuilder(LogLineDD orig) {
        this.level = orig.getLevel();
        this.date = orig.getDate();
        this.thread = orig.getThread();
        this.loggerName = orig.getLoggerName();
        this.message = orig.getMessage();
        this.stacktrace = orig.getStacktrace();
    }

    private static final ThrowableProxyConverter converter;
    static {
        converter = new ThrowableProxyConverter();
        converter.setOptionList(Arrays.asList("full"));
        converter.start();
    }
    public LogLineDDBuilder(ILoggingEvent loggingEvent) {
        this.level = loggingEvent.getLevel() == null ? null : loggingEvent.getLevel()+"";
        this.date = new Timestamp(loggingEvent.getTimeStamp());
        this.thread = loggingEvent.getThreadName();
        this.loggerName = loggingEvent.getLoggerName();
        this.message = loggingEvent.getMessage();
        if (loggingEvent.getThrowableProxy() != null) {
            this.stacktrace = converter.convert(loggingEvent);
        } else {
            this.stacktrace = null;
        }
    }

    public LogLineDDBuilder setLevel(String level) {
        this.level = level;
        return this;
    }

    public LogLineDDBuilder setDate(Timestamp date) {
        this.date = date;
        return this;
    }

    public LogLineDDBuilder setThread(String thread) {
        this.thread = thread;
        return this;
    }

    public LogLineDDBuilder setLoggerName(String loggerName) {
        this.loggerName = loggerName;
        return this;
    }

    public LogLineDDBuilder setMessage(String message) {
        this.message = message;
        return this;
    }

    public LogLineDDBuilder setStacktrace(String stacktrace) {
        this.stacktrace = stacktrace;
        return this;
    }

    public LogLineDD create() {
        return new LogLineDD(level, date, thread, loggerName, message, stacktrace);
    }

    public static List<LogLineDD> createAll(@Nullable List<LogLineDDBuilder> builders) {
        if (builders == null)
            return null;

        List<LogLineDD> res = new ArrayList<>();
        for (LogLineDDBuilder b : builders) {
            res.add(b.create());
        }
        return res;
    }

    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(create());
        } catch (JsonProcessingException e) {
            LOG.error("Exception converting "+getClass().getSimpleName()+" to JSON", e);
            return "Exception converting "+ getClass().getSimpleName() +" to JSON: "+e.getMessage();
        }
    }
}